
#include <chronoptics/tof/csf_reader.h>
#include <stdio.h>
#include <string.h>

#define CHECK_ERROR(error)                           \
  if (error) {                                       \
    printf("ERROR: %s\n", tof_error_message(error)); \
    tof_error_delete(error);                         \
    return -1;                                       \
  }

int main(int argc, char **argv) {
  if (argc != 2) {
    printf("Expected %s <CSF File>\n", argv[0]);
    return -1;
  }

  tof_error_t error = NULL;
  tof_csf_reader_t rdr = tof_csf_reader_new(argv[1], &error);
  CHECK_ERROR(error);

  size_t total_count = tof_csf_reader_number_of_frames(rdr, &error);
  CHECK_ERROR(error);

  for (size_t i = 0; i < total_count; i++) {
    tof_data_t frame = tof_csf_reader_read_frame(rdr, &error);
    CHECK_ERROR(error);

    enum tof_frame_type type = tof_data_frame_type(frame, &error);
    CHECK_ERROR(error);

    int32_t mat_type = tof_data_mat_type(frame, &error);
    CHECK_ERROR(error);

    printf("Frame %-3lu is of frame type %-i and has matrix type %i. ", i, type,
           mat_type);

    void *data = tof_data_data(frame, &error);
    CHECK_ERROR(error);

    int nrows = tof_data_rows(frame, &error);
    CHECK_ERROR(error);

    int ncols = tof_data_cols(frame, &error);
    CHECK_ERROR(error);

    switch (mat_type) {
      case 0: {
        uint8_t *ptr = (uint8_t *)data;
        printf("Middle pixel has value %i\n", ptr[ncols * (nrows / 2)]);
        break;
      }

      case 2: {
        uint16_t *ptr = (uint16_t *)data;
        printf("Middle pixel has value %i\n", ptr[ncols * (nrows / 2)]);
        break;
      }

      case 5: {
        float *ptr = (float *)data;
        printf("Middle pixel has value %f\n", ptr[ncols * (nrows / 2)]);
        break;
      }

      case 16: {
        uint8_t *ptr = (uint8_t *)data;
        uint16_t pos = ncols * (nrows / 2);
        printf("Middle pixel has values %i %i %i\n", ptr[pos * 3],
               ptr[pos * 3 + 1], ptr[pos * 3 + 2]);
        break;
      }

      default:
        printf("Type not implemented\n");
        break;
    }
  }

  tof_csf_reader_delete(rdr);
}